from .booking_entry import BookingEntryTitleConfig
from .entry_accounting_information import EntryAccountingInformationTitleConfig
from .invoice import InvoiceTitleConfig, ConsolidatedInvoiceTitleConfig
from .invoice_type import InvoiceTypeTitleConfig
