# Generated by Django 4.1.10 on 2023-08-30 11:20

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("wbaccounting", "0006_alter_invoice_status"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="invoice",
            options={
                "permissions": (
                    ("can_generate_invoice", "Can Generate Invoice"),
                    ("administrate_invoice", "Can administer Invoice"),
                ),
                "verbose_name": "Invoice",
                "verbose_name_plural": "Invoices",
            },
        ),
    ]
