# Generated by Django 4.1.9 on 2023-05-16 14:46

import django_fsm
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("wbaccounting", "0001_initial_squashed_squashed_0005_alter_bookingentry_counterparty_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="invoice",
            name="status",
            field=django_fsm.FSMField(
                choices=[
                    ("DRAFT", "Draft"),
                    ("PENDING", "Pending"),
                    ("EXTERNAL", "External"),
                    ("SENT", "Sent"),
                    ("PAID", "Paid"),
                    ("CANCELLED", "Cancelled"),
                ],
                default="DRAFT",
                max_length=50,
                verbose_name="Status",
            ),
        ),
    ]
